@@ -26,6 +26,7 @@
 #include <QPainter>
 #include <QGraphicsPixmapItem>
 #include <QGraphicsSceneMouseEvent>
+#include <QTimer>
 
 namespace Kigo {
 
@@ -34,16 +35,32 @@ namespace Kigo {
     const int dotPositions19[] = {3,3, 3,9, 3,15, 9,3, 9,9, 9,15, 15,3, 15,9, 15,15};
 
 GameScene::GameScene(Game *game, QObject *parent)
-    : QGraphicsScene(parent), m_game(game)
-    , m_showLabels(Preferences::showBoardLabels()), m_showHint(false)
+    : QGraphicsScene(parent)
+    , m_game(game)
+    , m_gamePopup()
+    , m_showLabels(Preferences::showBoardLabels())
+    , m_showHint(false)
+    , m_hintTimer(new QTimer(this))
     , m_showMoveNumbers(Preferences::showMoveNumbers())
-    , m_showPlacementMarker(true), m_showTerritory(false)
-    , m_boardSize(Preferences::boardSize()), m_placementMarkerItem(0)
+    , m_showPlacementMarker(true)
+    , m_showTerritory(false)
+    , m_boardRect()
+    , m_mouseRect()
+    , m_gridRect()
+    , m_cellSize(0)
+    , m_stonePixmapSize()
+    , m_placementMarkerPixmapSize()
+    , m_boardSize(Preferences::boardSize())
+    , m_placementMarkerItem(nullptr)
+    , m_stoneItems()
+    , m_hintItems()
+    , m_territoryItems()
 {
-    connect(m_game, SIGNAL(boardChanged()), this, SLOT(updateStoneItems()));
-    connect(m_game, SIGNAL(boardSizeChanged(int)), this, SLOT(changeBoardSize(int)));
-    connect(m_game, SIGNAL(currentPlayerChanged(Player)), this, SLOT(hideHint()));
-    connect(ThemeRenderer::self(), SIGNAL(themeChanged(QString)), this, SLOT(themeChanged()));
+    connect(m_game, &Game::boardChanged, this, &GameScene::updateStoneItems);
+    connect(m_game, &Game::boardSizeChanged, this, &GameScene::changeBoardSize);
+    connect(m_game, &Game::currentPlayerChanged, this, &GameScene::hideHint);
+    connect(ThemeRenderer::self(), &ThemeRenderer::themeChanged, this, &GameScene::themeChanged);
+    connect(m_hintTimer, &QTimer::timeout, this, &GameScene::hideHint);
 
     m_gamePopup.setMessageTimeout(3000);
     m_gamePopup.setHideOnMouseClick(true);
@@ -71,7 +88,7 @@ void GameScene::resizeScene(int width, int height)
         removeItem(m_placementMarkerItem);
     }
     m_placementMarkerPixmapSize = QSize(static_cast<int>(m_cellSize / 4), static_cast<int>(m_cellSize / 4));
-    m_placementMarkerItem = addPixmap(ThemeRenderer::self()->renderElement(ThemeRenderer::PlacementMarker, m_placementMarkerPixmapSize));
+    m_placementMarkerItem = addPixmap(ThemeRenderer::self()->renderElement(ThemeRenderer::Element::PlacementMarker, m_placementMarkerPixmapSize));
     m_placementMarkerItem->setVisible(false);
     m_placementMarkerItem->setZValue(1);
 }
@@ -86,6 +103,10 @@ void GameScene::showHint(bool show)
 {
     m_showHint = show;
     updateHintItems();
+
+    if (show == true) {
+        m_hintTimer->start(static_cast<int>(Preferences::hintVisibleTime() * 1000));
+    }
 }
 
 void GameScene::showMoveNumbers(bool show)
@@ -117,33 +138,33 @@ void GameScene::showTerritory(bool show)
 
 void GameScene::updateStoneItems()
 {
-    QGraphicsPixmapItem *item;
-    int halfStoneSize = m_stonePixmapSize.width() / 2;
+    QGraphicsPixmapItem *stoneItem;
+    const int halfStoneSize = m_stonePixmapSize.width() / 2;
 
     const Stone lastStone = (m_game->moves().size() > 0) ? m_game->lastMove().stone() : Stone::Invalid;
 
-    foreach (item, m_stoneItems) {  // Clear all stone items
-        removeItem(item);
+    foreach (stoneItem, m_stoneItems) {  // Clear all stone items
+        removeItem(stoneItem);
     }
     m_stoneItems.clear();
 
     foreach (const Stone &stone, m_game->stones(m_game->blackPlayer())) {
-        ThemeRenderer::Element element = (stone == lastStone) ? ThemeRenderer::BlackStoneLast : ThemeRenderer::BlackStone;
-        item = addPixmap(ThemeRenderer::self()->renderElement(element, m_stonePixmapSize));
-        item->setZValue(2);
-        int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
-        item->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfStoneSize + 1,
+        ThemeRenderer::Element element = (stone == lastStone) ? ThemeRenderer::Element::BlackStoneLast : ThemeRenderer::Element::BlackStone;
+        stoneItem = addPixmap(ThemeRenderer::self()->renderElement(element, m_stonePixmapSize));
+        stoneItem->setZValue(2);
+        const int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
+        stoneItem->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfStoneSize + 1,
                              m_gridRect.y() + (m_boardSize - stone.y()) * m_cellSize - halfStoneSize + 1));
-        m_stoneItems.append(item);
+        m_stoneItems.append(stoneItem);
     }
     foreach (const Stone &stone, m_game->stones(m_game->whitePlayer())) {
-        ThemeRenderer::Element element = (stone == lastStone) ? ThemeRenderer::WhiteStoneLast : ThemeRenderer::WhiteStone;
-        item = addPixmap(ThemeRenderer::self()->renderElement(element, m_stonePixmapSize));
-        item->setZValue(2);
-        int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
-        item->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfStoneSize + 1,
+        ThemeRenderer::Element element = (stone == lastStone) ? ThemeRenderer::Element::WhiteStoneLast : ThemeRenderer::Element::WhiteStone;
+        stoneItem = addPixmap(ThemeRenderer::self()->renderElement(element, m_stonePixmapSize));
+        stoneItem->setZValue(2);
+        const int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
+        stoneItem->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfStoneSize + 1,
                              m_gridRect.y() + (m_boardSize - stone.y()) * m_cellSize - halfStoneSize + 1));
-        m_stoneItems.append(item);
+        m_stoneItems.append(stoneItem);
     }
 
     if (m_showMoveNumbers) {
@@ -153,7 +174,7 @@ void GameScene::updateStoneItems()
             QPointF pos = QPointF(m_gridRect.x() + xOff * m_cellSize,
                                   m_gridRect.y() + (m_boardSize - move.stone().y()) * m_cellSize);
 
-            if (QGraphicsPixmapItem *item = static_cast<QGraphicsPixmapItem *>(itemAt(pos))) {
+            if (QGraphicsPixmapItem *item = static_cast<QGraphicsPixmapItem *>(itemAt(pos, QTransform()))) {
                 // We found an item in the scene that is in our move numbers, so we paint it's move number
                 // on top of the item and that's all.
                 //TODO: Check for existing move number to do special treatment
@@ -184,14 +205,14 @@ void GameScene::updateHintItems()
     m_hintItems.clear();
 
     if (m_showHint) {
-        int halfStoneSize = m_stonePixmapSize.width() / 2;
+        const int halfStoneSize = m_stonePixmapSize.width() / 2;
 
         foreach (const Stone &move, m_game->bestMoves(m_game->currentPlayer())) {
             QPixmap stonePixmap;
             if (m_game->currentPlayer().isWhite()) {
-                stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::WhiteStoneTransparent, m_stonePixmapSize);
+                stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::WhiteStoneTransparent, m_stonePixmapSize);
             } else if (m_game->currentPlayer().isBlack()) {
-                stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::BlackStoneTransparent, m_stonePixmapSize);
+                stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::BlackStoneTransparent, m_stonePixmapSize);
             }
 
             QPainter painter(&stonePixmap);
@@ -208,7 +229,7 @@ void GameScene::updateHintItems()
 
             item = addPixmap(stonePixmap);
             item->setZValue(4);
-            int xOff = move.x() >= 'I' ? move.x() - 'A' - 1 : move.x() - 'A';
+            const int xOff = move.x() >= 'I' ? move.x() - 'A' - 1 : move.x() - 'A';
             item->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfStoneSize + 2,
                                  m_gridRect.y() + (m_boardSize - move.y()) * m_cellSize - halfStoneSize + 2));
             m_hintItems.append(item);
@@ -227,24 +248,24 @@ void GameScene::updateTerritoryItems()
 
     if (m_showTerritory) {
         QPixmap stonePixmap;
-        int halfCellSize = m_cellSize / 2;
-        kDebug() << "Fetching territory from engine ...";
+        const int halfCellSize = m_cellSize / 2;
+        //qDebug() << "Fetching territory from engine ...";
 
-        stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::WhiteTerritory, QSize(m_cellSize, m_cellSize));
-        foreach (const Stone &stone, m_game->finalStates(Game::FinalWhiteTerritory)) {
+        stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::WhiteTerritory, QSize(m_cellSize, m_cellSize));
+        foreach (const Stone &stone, m_game->finalStates(Game::FinalState::FinalWhiteTerritory)) {
             item = addPixmap(stonePixmap);
             item->setZValue(8);
-            int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
+            const int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
             item->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfCellSize + 2,
                                  m_gridRect.y() + (m_boardSize - stone.y()) * m_cellSize - halfCellSize + 2));
             m_territoryItems.append(item);
         }
 
-        stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::BlackTerritory, QSize(m_cellSize, m_cellSize));
-        foreach (const Stone &stone, m_game->finalStates(Game::FinalBlackTerritory)) {
+        stonePixmap = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::BlackTerritory, QSize(m_cellSize, m_cellSize));
+        foreach (const Stone &stone, m_game->finalStates(Game::FinalState::FinalBlackTerritory)) {
             item = addPixmap(stonePixmap);
             item->setZValue(8);
-            int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
+            const int xOff = stone.x() >= 'I' ? stone.x() - 'A' - 1 : stone.x() - 'A';
             item->setPos(QPointF(m_gridRect.x() + xOff * m_cellSize - halfCellSize + 2,
                                  m_gridRect.y() + (m_boardSize - stone.y()) * m_cellSize - halfCellSize + 2));
             m_territoryItems.append(item);
@@ -272,11 +293,11 @@ void GameScene::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
         // Show a placement marker at the nearest board intersection
         // as a visual aid for the user.
         if (m_showPlacementMarker) {
-            int row = static_cast<int>((event->scenePos().x() - m_mouseRect.x()) / m_cellSize);
-            int col = static_cast<int>((event->scenePos().y() - m_mouseRect.y()) / m_cellSize);
+            const int row = static_cast<int>((event->scenePos().x() - m_mouseRect.x()) / m_cellSize);
+            const int col = static_cast<int>((event->scenePos().y() - m_mouseRect.y()) / m_cellSize);
 
-            int x = m_mouseRect.x() + row * m_cellSize + m_cellSize/2 - m_placementMarkerPixmapSize.width()/2;
-            int y = m_mouseRect.y() + col * m_cellSize + m_cellSize/2 - m_placementMarkerPixmapSize.height()/2;
+            const int x = m_mouseRect.x() + row * m_cellSize + m_cellSize/2 - m_placementMarkerPixmapSize.width()/2;
+            const int y = m_mouseRect.y() + col * m_cellSize + m_cellSize/2 - m_placementMarkerPixmapSize.height()/2;
 
             m_placementMarkerItem->setVisible(true);
             m_placementMarkerItem->setPos(x, y);
@@ -286,9 +307,9 @@ void GameScene::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
         if (m_game->currentPlayer().isHuman()) {
             if (m_game->currentPlayer().isWhite()) {
-                map = ThemeRenderer::self()->renderElement(ThemeRenderer::WhiteStoneTransparent, m_stonePixmapSize);
+                map = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::WhiteStoneTransparent, m_stonePixmapSize);
             } else if (m_game->currentPlayer().isBlack()) {
-                map = ThemeRenderer::self()->renderElement(ThemeRenderer::BlackStoneTransparent, m_stonePixmapSize);
+                map = ThemeRenderer::self()->renderElement(ThemeRenderer::Element::BlackStoneTransparent, m_stonePixmapSize);
             }
             emit cursorPixmapChanged(map);
         }
@@ -302,7 +323,7 @@ void GameScene::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
     if (m_mouseRect.contains(event->scenePos())) {
         int row = static_cast<int>((event->scenePos().x() - m_mouseRect.x()) / m_cellSize);
-        int col = static_cast<int>((event->scenePos().y() - m_mouseRect.y()) / m_cellSize);
+        const int col = static_cast<int>((event->scenePos().y() - m_mouseRect.y()) / m_cellSize);
         if (row < 0 || row >= m_boardSize || col < 0 || col >= m_boardSize) {
             return;
         }
@@ -318,15 +339,15 @@ void GameScene::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
 void GameScene::drawBackground(QPainter *painter, const QRectF &)
 {
-    ThemeRenderer::self()->renderElement(ThemeRenderer::Background, painter, sceneRect());
-    ThemeRenderer::self()->renderElement(ThemeRenderer::Board, painter, m_boardRect);
+    ThemeRenderer::self()->renderElement(ThemeRenderer::Element::Background, painter, sceneRect());
+    ThemeRenderer::self()->renderElement(ThemeRenderer::Element::Board, painter, m_boardRect);
 
-    int width = m_cellSize / 16;
-    QColor color = QColor(20, 30, 20);
+    const int width = m_cellSize / 16;
+    const QColor color = QColor(20, 30, 20);
     painter->setPen(QPen(color, width));
 
     for (int i = 0; i < m_boardSize; i++) {
-        qreal offset = i * m_cellSize;
+        const qreal offset = i * m_cellSize;
         painter->drawLine(QPointF(m_gridRect.left(),  m_gridRect.top() + offset),
                           QPointF(m_gridRect.right(), m_gridRect.top() + offset));
         painter->drawLine(QPointF(m_gridRect.left() + offset, m_gridRect.top()),
@@ -339,19 +360,19 @@ void GameScene::drawBackground(QPainter *painter, const QRectF &)
                 c = QChar('A' + i + 1);
             }
 
-            QString n = QString::number(m_boardSize - i);
+            const QString n = QString::number(m_boardSize - i);
             QFont f = painter->font();
             f.setPointSizeF(m_cellSize / 4);
             painter->setFont(f);
-            QFontMetrics fm = painter->fontMetrics();
+            const QFontMetrics fm = painter->fontMetrics();
 
             // Draw vertical numbers for board coordinates
-            qreal yVert = m_gridRect.top() + offset + fm.descent();
+            const qreal yVert = m_gridRect.top() + offset + fm.descent();
             painter->drawText(QPointF(m_gridRect.left() - m_cellSize + 2, yVert), n);
             painter->drawText(QPointF(m_gridRect.right() + m_cellSize - fm.width(n) - 3, yVert), n);
 
             // Draw horizontal characters for board coordinates
-            qreal xHor = m_gridRect.left() + offset - fm.width(c) / 2;
+            const qreal xHor = m_gridRect.left() + offset - fm.width(c) / 2;
             painter->drawText(QPointF(xHor, m_gridRect.top() - m_cellSize + fm.ascent() + 2), QString(c));
             painter->drawText(QPointF(xHor, m_gridRect.bottom() + m_cellSize - 3), QString(c));
         }
@@ -359,13 +380,13 @@ void GameScene::drawBackground(QPainter *painter, const QRectF &)
 
     // Draw thicker connections on some defined points.
     // This is extremely helpful to orientate oneself especially on the 19x19 board.
-    int radius = m_cellSize / 10;
+    const int radius = m_cellSize / 10;
     painter->setBrush(color);
     painter->setRenderHint(QPainter::Antialiasing);
 
     // in order to center properly we need to take line width into account
     // if the line has an odd width, we shift 1/5 pixel
-    qreal centerOffset = (width % 2) ? 0.5 : 0.0;
+    const qreal centerOffset = (width % 2) ? 0.5 : 0.0;
 
     // only do this for the common board sizes,
     // other sizes are a bit odd anyway
@@ -392,5 +413,3 @@ void GameScene::drawBackground(QPainter *painter, const QRectF &)
 }
 
 } // End of namespace Kigo
-
-#include "moc_gamescene.cpp"
